<?php
require_once '/var/packages/Spreadsheet/target/phpoffice/word/Autoloader.php';

use \PhpOffice\PhpWord\Style\ListItem as ListItemStyle;
\PhpOffice\PhpWord\Autoloader::register();
\PhpOffice\PhpWord\Settings::loadConfig();

// ===== for debugging =====================
$ob_file = fopen('/var/log/synoffice.log','a');

function ob_file_callback($buffer)
{
	if ($buffer !== '') {
		global $ob_file;
		fwrite($ob_file,"doc export: " . $buffer);
	}
}

ob_start('ob_file_callback');
//======================

function print_log($msg) {
	$line = "doc export@" . Date('Y-m-d H:i:s') . ": $msg\n";
	file_put_contents('/var/log/synoffice.log', $line, FILE_APPEND);
}

function print_log_array($ary) {
	foreach ($ary as $k => $v) {
		print_log("{$k}:{$v}");
	}
}

function arraySetValue($src, &$dst, $key, $defaultValue) {
	if (array_key_exists($key, $src)) {
		$dst[$key] = $src[$key];
	} else {
		$dst[$key] = $defaultValue;
	}
}

class SynoDocHtmlParser extends \PhpOffice\PhpWord\Shared\Html {
	/**
	 * Store the parsed table grid
	 *
	 * @todo Table inside table
	 */
	private static $grids = array();

	private static function findChild($node, $nodeName) {
		foreach ($node->childNodes as $child) {
			if ($child->nodeName === $nodeName) {
				return $child;
			}
		}
		return null;
	}

	private static function parseJsonData($attribute, $key, $styles)
	{
		$styles[$key] = json_decode($attribute->value, true);
		return $styles;
	}

	/**
	 * Parse more attribute like synohead, synochart, config
	 *
	 * @param \DOMAttr $attribute
	 * @return array
	 */
	protected static function parseInlineStyle($node, $styles = array())
	{
		global $comment;

		if ($node->nodeType == XML_ELEMENT_NODE) {
			$attributes = $node->attributes; // get all the attributes(eg: id, class)

			foreach ($attributes as $attribute) {
				switch ($attribute->name) {
				case 'style':
					$styles = static::parseStyle($attribute, $styles);
					break;
				case 'synohead':
				case 'synochart':
				case 'synograph':
				case 'synobr':
					$styles[$attribute->name] = $attribute->value;
					break;
				case 'data-syno-comment':
					$styles['synocomment'] = explode(' ', $attribute->value);

					foreach ($styles['synocomment'] as $id) {
						$c = $comment[$id];
						if ($c && $c['type'] === 1) {
							$styles['synosuggestion'] = (strpos($c['text'], 'deletion') !== false)? 'deletion' : 'insertion';
							break;
						}
					}
					break;
				case 'graph-config':
					$styles = static::parseJsonData($attribute, $attribute->name, $styles);
					if (is_null($styles['graph-config']['fill'])) {
						$styles['graph-config']['fill'] = '#CCE6FF';
					}
					if (is_null($styles['graph-config']['stroke'])) {
						$styles['graph-config']['stroke'] = '#2E3133';
					}
					break;
				case 'config':
					$styles = static::parseJsonData($attribute, $attribute->name, $styles);
					break;
				}
			}
		}

		return $styles;
	}

	private static function parsePageSize($type) {
		$sizes = array(
			"letter" => array("w"=>12240, "h"=>15840, "code"=>1),
			"tabloid" => array("w"=>15840, "h"=>24480, "code"=>3),
			"legal" => array("w"=>12240, "h"=>20160, "code"=>5),
			"statement" => array("w"=>7920, "h"=>12240, "code"=>6),
			"executive" => array("w"=>10440, "h"=>15120, "code"=>7),
			"folio" => array("w"=>12240, "h"=>18720, "code"=>14),
			"a3" => array("w"=>16838, "h"=>23811, "code"=>8),
			"a4" => array("w"=>11906, "h"=>16838, "code"=>9),
			"a5" => array("w"=>8391, "h"=>11906, "code"=>11),
			"b4" => array("w"=>14173, "h"=>20013, "code"=>33),
			"b5" => array("w"=>9978, "h"=>14173, "code"=>34)
		);
		if ($sizes[$type]) {
			return $sizes[$type];
		} else {
			return $sizes["letter"];
		}
	}

    /**
     * Parse div node
     *
     * @param \DOMNode $node
     * @param \PhpOffice\PhpWord\Element\AbstractContainer $element
     * @param array &$styles
     * @return \PhpOffice\PhpWord\Element
     */
	protected static function parseDiv($node, $element, &$styles)
	{
		if ($node->getAttribute('syno_invisible') === "true") {
			$jsonObj = json_decode($node->getAttribute('syno-layout'));
			$size = self::parsePageSize($jsonObj->{'page_type'});
			$element->getStyle()->setPageSizeW($size['w']);
			$element->getStyle()->setPageSizeH($size['h']);
			$element->getStyle()->setPaperCode($size['code']);
			$element->getStyle()->setOrientation($jsonObj->{'orientation'});
			$element->getStyle()->setMarginTop(round(\PhpOffice\PhpWord\Shared\Converter::cmToTwip($jsonObj->{'top'}), 0));
			$element->getStyle()->setMarginBottom(round(\PhpOffice\PhpWord\Shared\Converter::cmToTwip($jsonObj->{'bottom'}), 0));
			$element->getStyle()->setMarginLeft(round(\PhpOffice\PhpWord\Shared\Converter::cmToTwip($jsonObj->{'left'}), 0));
			$element->getStyle()->setMarginRight(round(\PhpOffice\PhpWord\Shared\Converter::cmToTwip($jsonObj->{'right'}), 0));
		}

		if ($node->getAttribute('synodecorate') === "true") {
			$synodeco = &$element->getPhpWord()->synoDecorator;
			$synodeco['place-show']  = ($node->getAttribute('place-show')  === "true");
			$synodeco['place-cover'] = ($node->getAttribute('place-cover') === "true");
			$synodeco['place-even']  = ($node->getAttribute('place-even')  === "true");
			$synodeco['num-show']    = ($node->getAttribute('num-show')    === "true");
			$synodeco['num-cover']   = ($node->getAttribute('num-cover')   === "true");
			$synodeco['num-even']    = ($node->getAttribute('num-even')    === "true");
			$synodeco['num-pos']     = $node->getAttribute('num-pos');
		}

		if ($node->getAttribute('data-syno-page-break') === "true") {
			$element->addPageBreak();
		}

		return $element;
	}

	/**
	 * Parse table node
	 *
	 * @param \DOMNode $node
	 * @param \PhpOffice\PhpWord\Element\AbstractContainer $element
	 * @param array &$styles
	 * @param string $argument1 Method name
	 * @return \PhpOffice\PhpWord\Element\AbstractContainer $element
	 *
	 * @todo As soon as TableItem, RowItem and CellItem support relative width and height
	 */
	protected static function parseTable($node, $element, &$styles, $argument1, $argument2) {
		global $comment;

		if ($argument1 === 'addRow') {
			// Does not matter?
			return null;
		}

		if ($argument1 === 'addCell') {
			// Find the owning table node
			$table = $node;
			while ($table && ($table->nodeName !== 'table')) {
				$table = $table->parentNode;
			}
			if (!$table) {
				return null;
			}

			// Pop stack
			while ((count(self::$grids) > 0) && (end(self::$grids)['node'] !== $table)) {
				array_pop(self::$grids);
			}
			if (count(self::$grids) < 1) {
				return null;
			}

			for ($i = 0, $numRows = count(end(self::$grids)['grid']); $i < $numRows; ++$i) {
				for ($j = 0, $numCols = count(end(self::$grids)['grid'][$i]['cells']); $j < $numCols; ++$j) {
					if (end(self::$grids)['grid'][$i]['cells'][$j]['node'] === $node) {
						return end(self::$grids)['grid'][$i]['cells'][$j]['element'];
					}
				}
			}
			return null;
		}

		$styles['paragraph'] = static::parseInlineStyle($node, $styles['paragraph']);
		if (array_key_exists('width', $styles['paragraph'])) {
			$argument2['width'] = $styles['paragraph']['width'] * 15;
			$argument2['unit'] = 'dxa';
		}

		// Export colgroup
		$colgroup = static::findChild($node, 'colgroup');
		if ($colgroup) {
			$width = 0;
			$argument2['colgroup'] = array();
			foreach ($colgroup->childNodes as $col) {
				$styles = static::parseInlineStyle($col);
				$argument2['colgroup'][] = static::parseInlineStyle($col);
				$width += (int)(end($argument2['colgroup'])['width']);
			}
			$argument2['width'] = $width * 15;
			$argument2['unit'] = 'dxa';
		}

		$newElement = $element->addTable($argument2);

		// Build table grid
		$grid = array();
		$tbody = static::findChild($node, 'tbody');
		foreach ($tbody->childNodes as $row => $tr) {
			$style = static::parseInlineStyle($tr);
			if (isset($style['height'])) {
				$height = (int)$style['height'] * 15;
			}
			if (array_key_exists('synocomment', $style) && array_key_exists('synosuggestion', $style)) {
				$comment[] = array('type' => 1);
				$c = $comment[$style['synocomment'][0]];
				$revision = array(
					'id'     => $c['id'],
					'@id'    => count($comment),
					'op'     => array('deletion' => 'del', 'insertion' => 'ins')[$style['synosuggestion']],
					'author' => $c['author'],
					'date'   => $c['date'],
				);
			}
			$col = 0;
			foreach ($tr->childNodes as $td) {
				if (isset($grid[$row])) {
					while (isset($grid[$row]['cells'][$col])) {
						++$col;
					}
				}
				$rowspan = ($td->hasAttribute('rowspan') ? (int)$td->getAttribute('rowspan') : 1);
				$colspan = ($td->hasAttribute('colspan') ? (int)$td->getAttribute('colspan') : 1);
				$style = static::parseInlineStyle($td);
				if (array_key_exists('colgroup', $argument2)) {
					$colgroup = $argument2['colgroup'];
					$width = 0;
					for ($i = 0; $i < $colspan; ++$i) {
						$width += (int)$colgroup[$col + $i]['width'] * 15;
					}
				} elseif (isset($style['width'])) {
					$width = (int)$style['width'] * 15;
				}
				if (!isset($grid[$row])) {
					$grid[$row] = array(
						'node' => null,
						'height' => null,
						'cells' => array(),
						'revision' => null);
				}
				$grid[$row]['cells'][$col] = array(
					'node' => $td,
					'width' => $width,
					'rowspan' => $rowspan,
					'colspan' => $colspan);
				$defaultBorderStyles = array(
					'borderLeftColor' => 'aaaaaa', 'borderLeftStyle' => 'single', 'borderLeftWidth' => 8,
					'borderTopColor' => 'aaaaaa', 'borderTopStyle' => 'single', 'borderTopWidth' => 8,
					'borderRightColor' => 'aaaaaa', 'borderRightStyle' => 'single', 'borderRightWidth' => 8,
					'borderBottomColor' => 'aaaaaa', 'borderBottomStyle' => 'single', 'borderBottomWidth' => 8,
					'bgColor' => 'ffffff', 'VAlign' => 'center');
				foreach ($defaultBorderStyles as $key => $value) {
					arraySetValue($style, $grid[$row]['cells'][$col], $key, $value);
				}
			}
			$grid[$row]['node'] = $tr;
			if ($height) {
				$grid[$row]['height'] = $height;
			}
			if ($revision) {
				$grid[$row]['revision'] = $revision;
			}
		}

		// Create row and cell elements
		$table = $newElement;
		for ($i = 0, $numRows = count($grid); $i < $numRows; ++$i) {
			$tr = $table->addRow($grid[$i]['height']);
			$tr->setRevision($grid[$i]['revision']);
			for ($j = 0, $numCols = count($grid[$i]['cells']); $j < $numCols; ++$j) {
				for ($k = 1; $k < $grid[$i]['cells'][$j]['rowspan']; $k++) {
					$grid[$i + $k]['cells'][$j]['rowspan'] = -1;
				}
				if ($grid[$i]['cells'][$j]['colspan'] < 0) {
					$grid[$i]['cells'][$j]['element'] = $grid[$i]['cells'][$j - 1]['element'];
					continue;
				}
				unset($rowspan);
				unset($colspan);
				if ($grid[$i]['cells'][$j]['rowspan'] > 1) {
					$rowspan = 'restart';
					for ($k = 1; $k < $grid[$i]['cells'][$j]['rowspan']; $k++) {
						if (!isset($grid[$i + $k])) {
							break;
						}
						if (!isset($grid[$i + $k]['cells'][$j])) {
							continue;
						}
						$grid[$i + $k]['cells'][$j]['rowspan'] = -1;
						$grid[$i + $k]['cells'][$j]['colspan'] = $grid[$i]['cells'][$j]['colspan'];
					}
				} else if ($grid[$i]['cells'][$j]['rowspan'] < 0) {
					$rowspan = 'continue';
				}
				if ($grid[$i]['cells'][$j]['colspan'] > 1) {
					$colspan = $grid[$i]['cells'][$j]['colspan'];
				}
				$td = $tr->addCell($grid[$i]['cells'][$j]['width'], array(
					'VMerge' => $rowspan,
					'GridSpan' => $colspan,
					'borderLeftColor' => $grid[$i]['cells'][$j]['borderLeftColor'],
					'borderLeftStyle' => $grid[$i]['cells'][$j]['borderLeftStyle'],
					'borderLeftSize' => $grid[$i]['cells'][$j]['borderLeftWidth'],
					'borderTopColor' => $grid[$i]['cells'][$j]['borderTopColor'],
					'borderTopStyle' => $grid[$i]['cells'][$j]['borderTopStyle'],
					'borderTopSize' => $grid[$i]['cells'][$j]['borderTopWidth'],
					'borderRightColor' => $grid[$i]['cells'][$j]['borderRightColor'],
					'borderRightStyle' => $grid[$i]['cells'][$j]['borderRightStyle'],
					'borderRightSize' => $grid[$i]['cells'][$j]['borderRightWidth'],
					'borderBottomColor' => $grid[$i]['cells'][$j]['borderBottomColor'],
					'borderBottomStyle' => $grid[$i]['cells'][$j]['borderBottomStyle'],
					'borderBottomSize' => $grid[$i]['cells'][$j]['borderBottomWidth'],
					'bgColor' => $grid[$i]['cells'][$j]['bgColor'],
					'VAlign' => $grid[$i]['cells'][$j]['VAlign']));
				$grid[$i]['cells'][$j]['element'] = $td;
			}
		}

		self::$grids[] = array('node' => $node, 'grid' => $grid);
		return $newElement;
	}

	/**
	 * Parse image node
	 *
	 * @param \DOMNode $node
	 * @param \PhpOffice\PhpWord\Element\AbstractContainer $element
	 * @return \PhpOffice\PhpWord\Element\TextRun
	 *
	 */
	protected static function parseImage($node, $element, &$styles) {
		global $attachment;

		$style = array();
		foreach ($node->attributes as $attribute) {
			switch ($attribute->name) {
				case 'alt': {
					$style['synoAlt'] = $attribute->value;
					break;
				}
				case 'file-id': {
					if (array_key_exists($attribute->value, $attachment)) {
						$file = $attachment[$attribute->value];
						$src = $file['path'];
						$filename = $file['basename'].".".$file['ext'];
						$style['synoFileId'] = $attribute->value;
					}
					break;
				}
				case 'file-type': {
					$style['synoFileType'] = $attribute->value;
					break;
				}
				case 'style': {
					$styleattr = explode(';', $attribute->value);
					foreach ($styleattr as $attr) {
						if (!strpos($attr, ':')) {
							continue;
						}
						list($k, $v) = explode(':', $attr);
						switch (trim($k)) {
							case 'width': {
								$style['width'] = (int)trim($v);
								break;
							}
							case 'height': {
								$style['height'] = (int)trim($v);
								break;
							}
							// case 'float': {
							// 	if (trim($v) == 'right') {
							// 		$style['hPos'] = \PhpOffice\PhpWord\Style\Image::POS_RIGHT;
							// 		$style['hPosRelTo'] = \PhpOffice\PhpWord\Style\Image::POS_RELTO_PAGE;
							// 		$style['pos'] = \PhpOffice\PhpWord\Style\Image::POS_RELATIVE;
							// 		$style['wrap'] = \PhpOffice\PhpWord\Style\Image::WRAP_TIGHT;
							// 		$style['overlap'] = true;
							// 	}
							// 	if (trim($v) == 'left') {
							// 		$style['hPos'] = \PhpOffice\PhpWord\Style\Image::POS_LEFT;
							// 		$style['hPosRelTo'] = \PhpOffice\PhpWord\Style\Image::POS_RELTO_PAGE;
							// 		$style['pos'] = \PhpOffice\PhpWord\Style\Image::POS_RELATIVE;
							// 		$style['wrap'] = \PhpOffice\PhpWord\Style\Image::WRAP_TIGHT;
							// 		$style['overlap'] = true;
							// 	}
							// 	break;
							// }
							default: {
								break;
							}
						}
					}
					break;
				}
				default: {
					break;
				}
			}
		}

		$newElement = null;

		switch ($style["synoFileType"]) {
			// FIXME: no image comes here
			case 'image':
				try {
					$newElement = $element->addImage($src, $style);
				} catch (Exception $e) {
					try {
						$src = '/var/packages/Spreadsheet/target/ui/images/_Asset/1x/_Common/icon_image_broken.png';
						$newElement = $element->addImage($src, $style);
						$newElement->setBroken(true);
					} catch (Exception $e) {
						syslog(LOG_ERR, $e->getMessage());
					}
				}
				if ($newElement) {
					$newElement->setAnnotations($styles['font']);
				}
				break;
			case 'binary':
				try {
					$info = pathinfo($src);
					$ole_path = $info['dirname'] . "/". $filename;
					\PhpOffice\PhpWord\Shared\OLE::convertFileToOLE($src, $ole_path);
					$element->addObject($ole_path, $style);
					$text = $element->addText($filename);
					$text->hide();
				} catch (Exception $e) {
					syslog(LOG_ERR, $e->getMessage());
				}
				break;
			default:
				echo "Unrecognized file type " . $style["synoFileType"] . "\n";
				break;
		}
		return $newElement;
	}

	/**
	 * Parse heading node
	 *
	 * @param \PhpOffice\PhpWord\Element\AbstractContainer $element
	 * @param array &$styles
	 * @param string $argument1 Name of heading style
	 * @return \PhpOffice\PhpWord\Element\TextRun
	 *
	 * @todo Think of a clever way of defining header styles, now it is only based on the assumption, that
	 * Heading1 - Heading6 are already defined somewhere
	 */
	protected static function parseHeading($node, $element, &$styles, $argument1, $argument2) {
		$styles['paragraph'] = static::parseInlineStyle($node, $styles['paragraph']);
		$styles['paragraph']['styleName'] = $argument1;
		self::setDefaultHeadingStyle($styles);
		$pStyle = $element->getParagraphStyle();
		if ($pStyle instanceof \PhpOffice\PhpWord\Style\Paragraph) {
			$pStyle->setStyleName($argument1);
		}
		$phpword = $element->getPhpWord();
		if ($phpword instanceof \PhpOffice\PhpWord\PhpWord) {
			$newElement = $element->addTitleRun($styles['paragraph'], $argument2, $styles['paragraph']['synohead'], $phpword);
		} else {
			$newElement = $element->addTextRun($styles['paragraph']);
		}
		return $newElement;
	}

	/**
	 * Set default value to spaceBefore, spaceAfter, lineHeight of Heading
	 *
	 */
	protected static function setDefaultHeadingStyle(&$styles) {
		$spaceBefore = 0;
		$spaceAfter = 0;
		$lineHeight = 1.5;
		switch ($styles['paragraph']['styleName']) {
			case "Heading1":
				$spaceBefore = 400;
				$spaceAfter = 120;
				break;
			case "Heading2":
				$spaceBefore = 320;
				$spaceAfter = 100;
				break;
			case "Heading3":
				$spaceBefore = 260;
				$spaceAfter = 80;
				break;
			case "Heading4":
				$spaceBefore = 200;
				$spaceAfter = 60;
				break;
			case "Heading5":
				$spaceBefore = 180;
				$spaceAfter = 60;
				break;
		}
		if ($styles['paragraph']['spaceBefore'] === null) {
			$styles['paragraph']['spaceBefore'] = $spaceBefore;
		}
		if ($styles['paragraph']['spaceAfter'] === null) {
			$styles['paragraph']['spaceAfter'] = $spaceAfter;
		}
		if ($styles['paragraph']['lineHeight'] === null) {
			$styles['paragraph']['lineHeight'] = $lineHeight;
		}
		return;
	}

	/**
	 * Get image style for adding a image
	 *
	 * @param array $graphConfig
	 * @return array
	 */
	protected static function getImageStyle($graphConfig) {
		global $attachment;
		$style = array();
		if (array_key_exists('file_alt', $graphConfig)) {
			$style['synoAlt'] = $graphConfig['file_alt'];
		}
		if (array_key_exists('file_id', $graphConfig)) {
			if (array_key_exists($graphConfig['file_id'], $attachment)) {
				$style['synoFileId'] = $graphConfig['file_id'];
			}
		}
		if (array_key_exists('file_type', $graphConfig)) {
			$style['synoFileType'] = $graphConfig['file_type'];
		}
		if (array_key_exists('width', $graphConfig)) {
			$style['width'] = $graphConfig['width'];
		}
		if (array_key_exists('height', $graphConfig)) {
			$style['height'] = $graphConfig['height'];
		}
		return $style;
	}

	/**
	 * Get image src for adding a image
	 *
	 * @param array $graphConfig
	 * @return string
	 */
	protected static function getImageSrc($graphConfig) {
		global $attachment;
		$src = '/var/packages/Spreadsheet/target/ui/images/_Asset/1x/_Common/icon_image_broken.png';
		if (!array_key_exists('file_id', $graphConfig)) {
			return $src;
		}
		$file_id = $graphConfig['file_id'];
		if (!array_key_exists($file_id, $attachment)) {
			return $src;
		}
		$file = $attachment[$file_id];
		$src = $file['path'];
		return $src;
	}

	/**
	 * Parse image, handle svg image
	 *
	 * @param \PhpOffice\PhpWord\Element\AbstractContainer $element
	 * @param array $graphConfig
	 * @return \PhpOffice\PhpWord\Element\TextRun
	 */
	protected static function parseSynoImage($element, $graphConfig) {
		global $attachment;
		global $floatingImages;
		$newElement = null;
		if (!array_key_exists('file_type', $graphConfig)) {
			return null;
		}
		$src = self::getImageSrc($graphConfig);
		switch ($graphConfig["file_type"]) {
			case 'image':
				$imageStyle = self::getImageStyle($graphConfig);
				$imageStyle['synoFileId'] = $graphConfig['file_id'];
				$imageStyle['synoFileType'] = $graphConfig['file_type'];
				$imageStyle['synoAlt'] = $graphConfig['file_alt'];
				if ($element instanceof \PhpOffice\PhpWord\Element\Section) {
					$floatingImages[] = array(
						'src' => $src,
						'imageStyle' => $imageStyle,
						'waterMark' => false,
						'graphConfig' => $graphConfig
					);
					return null;
				}
				try {
					$newElement = $element->addImage($src, $imageStyle, false, $graphConfig);
				} catch (Exception $e) {
					try {
						$src = '/var/packages/Spreadsheet/target/ui/images/_Asset/1x/_Common/icon_image_broken.png';
						$newElement = $element->addImage($src, $imageStyle);
						$newElement->setBroken(true);
					} catch (Exception $e) {
						syslog(LOG_ERR, $e->getMessage());
					}
				}
				break;
			// FIXME: no binary comes here
			case 'binary':
				$file = $attachment[$graphConfig['file_id']];
				$info = pathinfo($src);
				$src = $file['path'];
				$filename = $file['basename'].".".$file['ext'];
				$ole_path = $info['dirname'] . "/". $filename;
				\PhpOffice\PhpWord\Shared\OLE::convertFileToOLE($src, $ole_path);
				$element->addObject($ole_path);
				$text = $element->addText($filename);
				$text->hide();
				break;
			default:
				echo "Unrecognized file type " . $style["synoFileType"] . "\n";
				break;
		}
		return $newElement;
	}

	/**
	 * Parse span node, handle synochart, synograph, synobr
	 *
	 * @param \DOMNode $node
	 * @param \PhpOffice\PhpWord\Element\AbstractContainer $element
	 * @param array &$styles
	 * @return \PhpOffice\PhpWord\Element\TextRun
	 */
	protected static function parseSpan($node, $element, &$styles)
	{
		global $floatingGraphs;

		$inlineStyle = self::parseInlineStyle($node);

		if (array_key_exists('synochart', $inlineStyle) && array_key_exists('config', $inlineStyle)) {
			$config = $inlineStyle['config'];
			$chartType = $config['plotArea']['subCharts'][0]['type'];

			$chart = $element->addChart($chartType, array(), array());
			$chart->setAnnotations($styles['font']);
			$chart->setSynoChartConfig($config);
			if (empty($inlineStyle['width']) || empty($inlineStyle['height'])) {
				$inlineStyle['width'] = 520;
				$inlineStyle['height'] = 380;
			}
			$chart->setTitle($config['title']);
			$chart->setXAxisTitle($config['plotArea']['xAxisTitle']);
			$chart->setYAxisTitle($config['plotArea']['yAxisTitle']);
			$chart->getStyle()->setWidth(\PhpOffice\PhpWord\Shared\Converter::pixelToEmu($inlineStyle['width']));
			$chart->getStyle()->setHeight(\PhpOffice\PhpWord\Shared\Converter::pixelToEmu($inlineStyle['height']));
			$styles['paragraph'] = array_merge($inlineStyle, $styles['paragraph']);
			return $element;
		}

		if (array_key_exists('synograph', $inlineStyle)) {
			if (!array_key_exists('graph-config', $inlineStyle) ||
				!array_key_exists('type', $inlineStyle['graph-config'])) {
				return null;
			}
			$graph_type = $inlineStyle['graph-config']['type'];
			if ('image' === $graph_type) {
				$newElement = self::parseSynoImage($element, $inlineStyle['graph-config']);
			} else {
				if ($element instanceof \PhpOffice\PhpWord\Element\Section) {
					$floatingGraphs[] = $inlineStyle['graph-config'];
					return null;
				}
				$newElement = $element->addGraph($inlineStyle['graph-config']);
			}
			if ($newElement) {
				$newElement->setAnnotations(array_merge($styles['font'], $inlineStyle));
			}
			return $newElement;
		}

		if (array_key_exists('synobr', $inlineStyle)) {
			$element->addTextBreak();
			return $element;
		}

		$styles['font'] = array_merge($inlineStyle, $styles['font']);
		return null;
	}
}

function generateRandomHex($length) {
	return strtoupper(substr(md5(rand()), 0, $length));
}

function readDocument($path) {
	global $attachment;
	global $comment;

	$content = file_get_contents($path, false);
	if ($content === false) {
		return '';
	}
	$json = json_decode($content, true);
	if ($json === null) {
		return '';
	}
	$attachment = $json['attachment'];
	$comment = array();
	foreach ($json['comment']['data'] as $c) {
		// FIXME: 8-digit hex may overflow?
		$c['paraId'] = '5' . generateRandomHex(7);
		$c['@id'] = count($comment);
		$c['date'] = date("Y-m-d\TH:i:s\Z", (int)($c['timestamp'] / 1000));
		$c['parent'] = null;
		$comment[$c['id']] = $c;

		foreach ($c['replies'] as $r) {
			// FIXME: 8-digit hex may overflow?
			$r['paraId'] = '6' . generateRandomHex(7);
			$r['@id'] = count($comment);
			$r['date'] = date("Y-m-d\TH:i:s\Z", (int)($c['timestamp'] / 1000));
			$r['parent'] = $c['id'];
			$comment[$r['id']] = $r;
		}
	}
	return $json;
}

function setHeadingStyle($phpword) {
	// TODO read heading style settings
	$phpword->addTitleStyle(1, array('size' => 24, 'color' => '000000'), array('lineHeight' => 1.5));
	$phpword->addTitleStyle(2, array('size' => 20, 'color' => '000000'), array('lineHeight' => 1.5));
	$phpword->addTitleStyle(3, array('size' => 16, 'color' => '000000'), array('lineHeight' => 1.5));
	$phpword->addTitleStyle(4, array('size' => 12, 'color' => '000000'), array('lineHeight' => 1.5));
	$phpword->addTitleStyle(5, array('size' => 11, 'color' => '555555'), array('lineHeight' => 1.5));
}

function setNumberingStyle($phpword)
{
	// Property mapping for numbering level information
	$properties = array('start', 'format', 'text', 'align', 'tabPos', 'left', 'hanging', 'font', 'hint');

	// Legacy level information
	$listTypeStyles = array(
		ListItemStyle::TYPE_SQUARE_FILLED => array(
			'type' => 'hybridMultilevel',
			'levels' => array(
				0 => '1, bullet, , left, 720, 720, 360, Wingdings, default',
				1 => '1, bullet, o, left, 1440, 1440, 360, Courier New, default',
				2 => '1, bullet, , left, 2160, 2160, 360, Wingdings, default',
				3 => '1, bullet, , left, 2880, 2880, 360, Symbol, default',
				4 => '1, bullet, o, left, 3600, 3600, 360, Courier New, default',
				5 => '1, bullet, , left, 4320, 4320, 360, Wingdings, default',
				6 => '1, bullet, , left, 5040, 5040, 360, Symbol, default',
				7 => '1, bullet, o, left, 5760, 5760, 360, Courier New, default',
				8 => '1, bullet, , left, 6480, 6480, 360, Wingdings, default',
			),
		),
		ListItemStyle::TYPE_BULLET_FILLED => array(
			'type' => 'hybridMultilevel',
			'levels' => array(
				0 => '1, bullet, , left, 720, 720, 360, Symbol, default',
				1 => '1, bullet, o, left, 1440, 1440, 360, Courier New, default',
				2 => '1, bullet, , left, 2160, 2160, 360, Wingdings, default',
				3 => '1, bullet, , left, 2880, 2880, 360, Symbol, default',
				4 => '1, bullet, o, left, 3600, 3600, 360, Courier New, default',
				5 => '1, bullet, , left, 4320, 4320, 360, Wingdings, default',
				6 => '1, bullet, , left, 5040, 5040, 360, Symbol, default',
				7 => '1, bullet, o, left, 5760, 5760, 360, Courier New, default',
				8 => '1, bullet, , left, 6480, 6480, 360, Wingdings, default',
			),
		),
		ListItemStyle::TYPE_BULLET_EMPTY => array(
			'type' => 'hybridMultilevel',
			'levels' => array(
				0 => '1, bullet, o, left, 720, 720, 360, Courier New, default',
				1 => '1, bullet, o, left, 1440, 1440, 360, Courier New, default',
				2 => '1, bullet, , left, 2160, 2160, 360, Wingdings, default',
				3 => '1, bullet, , left, 2880, 2880, 360, Symbol, default',
				4 => '1, bullet, o, left, 3600, 3600, 360, Courier New, default',
				5 => '1, bullet, , left, 4320, 4320, 360, Wingdings, default',
				6 => '1, bullet, , left, 5040, 5040, 360, Symbol, default',
				7 => '1, bullet, o, left, 5760, 5760, 360, Courier New, default',
				8 => '1, bullet, , left, 6480, 6480, 360, Wingdings, default',
			),
		),
		ListItemStyle::TYPE_NUMBER => array(
			'type' => 'hybridMultilevel',
			'levels' => array(
				0 => '1, decimal, %1., left, 810, 810, 360, , default',
				1 => '1, decimal, %2., left, 1260, 1260, 360, , default',
				2 => '1, decimal, %3., left, 1710, 1710, 360, , default',
				3 => '1, decimal, %4., left, 2160, 2160, 360, , default',
				4 => '1, decimal, %5., left, 2610, 2610, 360, , default',
				5 => '1, decimal, %6., left, 3060, 3060, 360, , default',
				6 => '1, decimal, %7., left, 3510, 3510, 360, , default',
				7 => '1, decimal, %8., left, 3960, 3960, 360, , default',
				8 => '1, decimal, %9., left, 4410, 4410, 360, , default',
				// 0 => '1, decimal, %1., left, 720, 720, 360, , default',
				// 1 => '1, bullet, o, left, 1440, 1440, 360, Courier New, default',
				// 2 => '1, bullet, , left, 2160, 2160, 360, Wingdings, default',
				// 3 => '1, bullet, , left, 2880, 2880, 360, Symbol, default',
				// 4 => '1, bullet, o, left, 3600, 3600, 360, Courier New, default',
				// 5 => '1, bullet, , left, 4320, 4320, 360, Wingdings, default',
				// 6 => '1, bullet, , left, 5040, 5040, 360, Symbol, default',
				// 7 => '1, bullet, o, left, 5760, 5760, 360, Courier New, default',
				// 8 => '1, bullet, , left, 6480, 6480, 360, Wingdings, default',
			),
		),
		ListItemStyle::TYPE_NUMBER_NESTED => array(
			'type' => 'multilevel',
			'levels' => array(
				0 => '1, decimal, %1., left, 360, 360, 360, , ',
				1 => '1, decimal, %1.%2., left, 792, 792, 432, , ',
				2 => '1, decimal, %1.%2.%3., left, 1224, 1224, 504, , ',
				3 => '1, decimal, %1.%2.%3.%4., left, 1800, 1728, 648, , ',
				4 => '1, decimal, %1.%2.%3.%4.%5., left, 2520, 2232, 792, , ',
				5 => '1, decimal, %1.%2.%3.%4.%5.%6., left, 2880, 2736, 936, , ',
				6 => '1, decimal, %1.%2.%3.%4.%5.%6.%7., left, 3600, 3240, 1080, , ',
				7 => '1, decimal, %1.%2.%3.%4.%5.%6.%7.%8., left, 3960, 3744, 1224, , ',
				8 => '1, decimal, %1.%2.%3.%4.%5.%6.%7.%8.%9., left, 4680, 4320, 1440, , ',
			),
		),
		ListItemStyle::TYPE_ALPHANUM => array(
			'type' => 'multilevel',
			'levels' => array(
				0 => '1, decimal, %1., left, 720, 720, 360, , ',
				1 => '1, lowerLetter, %2., left, 1440, 1440, 360, , ',
				2 => '1, lowerRoman, %3., right, 2160, 2160, 180, , ',
				3 => '1, decimal, %4., left, 2880, 2880, 360, , ',
				4 => '1, lowerLetter, %5., left, 3600, 3600, 360, , ',
				5 => '1, lowerRoman, %6., right, 4320, 4320, 180, , ',
				6 => '1, decimal, %7., left, 5040, 5040, 360, , ',
				7 => '1, lowerLetter, %8., left, 5760, 5760, 360, , ',
				8 => '1, lowerRoman, %9., right, 6480, 6480, 180, , ',
			),
		),
	);

	$arr = array(ListItemStyle::TYPE_BULLET_FILLED, ListItemStyle::TYPE_NUMBER);
	foreach ($arr as $type) {
		$numStyle = "PHPWordList{$type}";
		// Populate style and register to global Style register
		$style = $listTypeStyles[$type];
		foreach ($style['levels'] as $key => $value) {
			$level = array();
			$levelProperties = explode(', ', $value);
			$level['level'] = $key;
			for ($i = 0; $i < count($properties); $i++) {
				$property = $properties[$i];
				$level[$property] = $levelProperties[$i];
			}
			$style['levels'][$key] = $level;
		}
		$phpword->addNumberingStyle($numStyle, $style);
	}
}

function initPhpWord() {
	global $floatingGraphs;

	$floatingGraphs = array();
	\PhpOffice\PhpWord\Settings::setDefaultFontSize(11);
	$phpword = new \PhpOffice\PhpWord\PhpWord();
	$phpword->synoDecorator = array();
	setNumberingStyle($phpword);
	setHeadingStyle($phpword);
	return $phpword;
}

function parseHtml($phpword, $html) {
	$section = $phpword->addSection();
	SynoDocHtmlParser::addHtml($section, $html);
}

function saveDocx($phpword, $path) {
	$phpword->save($path, 'Word2007');
	chown($path, 'office');
	chgrp($path, 'office');
}

function main($argv) {
	$src = $argv[1];
	$dst = $argv[2];
	// $lang = $argv[3];

	$json = readDocument($src);
	$html = $json['content']['html'];
	//////file_put_contents('/var/log/export.json', json_encode($json));
	//////file_put_contents('/var/log/export.html', $html);
	$phpword = initPhpWord();
	parseHtml($phpword, $html);
	saveDocx($phpword, $dst);

	// embed syno doc's data into docx file
	if ( !empty($json['content']) && !empty($json['content']['data']) ) {
		$zip = new ZipArchive();
		$zip->open($dst);
		$zip->addFile($src, 'synoDoc.xml');
		$zip->close();
	}
}

main($argv);
?>
