from openpyxl.worksheet._read_only import ReadOnlyWorksheet
from openpyxl.utils.cell import coordinate_to_tuple
from .worksheet_parser import SynoWorkSheetParser
from .utils import boundaries_to_index


#pylint: disable=R0902

class SynoReadOnlyWorksheet:

    def __init__(self, parent_workbook, title, worksheet_path, shared_strings):
        self.parent = parent_workbook
        self.title = title
        self.sheet_state = 'visible'
        self._worksheet_path = worksheet_path
        self._shared_strings = shared_strings
        self._parser = None
        self.comments = {}
        self.charts = []
        self.images = []
        self._rels = None

        # pylint: disable=no-value-for-parameter
        self._get_source = ReadOnlyWorksheet._get_source.__get__(self)

    def get_bounds(self):
        src = self._get_source()
        parser = SynoWorkSheetParser(
            src, self._shared_strings, workbook=self.parent)
        bounds = parser.parse_dimensions()
        src.close()
        return boundaries_to_index(bounds) if bounds else None

    def iter_rows(self):
        src = self._get_source()
        self._parser = SynoWorkSheetParser(
            src, self._shared_strings, workbook=self.parent)
        for _, row in self._parser.parse():
            yield row
        src.close()

    def set_comment(self, ref, content):
        row, col = coordinate_to_tuple(ref)
        row -= 1
        col -= 1
        if not row in self.comments:
            self.comments[row] = {}
        self.comments[row][col] = content

    def __get_parsed_data(self, key):
        if self._parser is None:
            return None
        return getattr(self._parser, key, None)

    @property
    def merged_cells(self):
        return self.__get_parsed_data('merged_cells')

    @property
    def column_dimensions(self):
        return self.__get_parsed_data('column_dimensions')

    @property
    def row_dimensions(self):
        return self.__get_parsed_data('row_dimensions')

    @property
    def views(self):
        return self.__get_parsed_data('views')

    @property
    def sheet_properties(self):
        return self.__get_parsed_data('sheet_properties')

    @property
    def formatting(self):
        return self.__get_parsed_data('formatting')

    @property
    def auto_filter(self):
        return self.__get_parsed_data('auto_filter')

    @property
    def data_validations(self):
        return self.__get_parsed_data('data_validations')

    @property
    def hyperlinks(self):
        return self.__get_parsed_data('hyperlinks')

    @property
    def rows(self):
        return self.iter_rows()
