<?php

namespace PhpOffice\PhpPresentation\Writer\ProseMirror;

use PhpOffice\PhpPresentation\Style\Alignment;
use PhpOffice\PhpPresentation\Style\Bullet;
use PhpOffice\PhpPresentation\Style\Fill;
use PhpOffice\PhpPresentation\Shape\RichText;

class WithTextBox extends FloatingObject
{
    protected function setContent()
    {
        $shape = $this->createShape($this->phpShape);
        $shape->setShapeType($this->phpShape->getShapeType());
        $shape->setCustPoints($this->phpShape->getCustPoints());
        $this->setShapePathConfig($shape);

        $textBlock = $this->createTextBlock();
        $this->setTextBlockVAlign($textBlock);
        $this->setTextBlockPadding($textBlock);

        $isSlideHolder = (false !== strpos($this->type, 'slide_holder')) ? true : false;

        $phpParagraphs = $this->phpShape->getParagraphs();

        $preType = Bullet::TYPE_NOTSET;
        $preSubType = '';
        $levelIndentMapping = array();
        $listStack = array();
        foreach ($phpParagraphs as $phpParagraph) {
            $noText = false;
            if (0 === count($phpParagraph->getRichTextElements())) {
                $noText = true;
            }
            $paragraph = new Paragraph();
            $paragraph->setSlide($this->slide);

            $alignment = $phpParagraph->getAlignment();
            $level = $alignment->getLevel() + 1;
            $bulletType = Bullet::TYPE_NOTSET;

            $marginLeft = (Alignment::MARGIN_NONE === $alignment->getMarginLeft()) ? 0 : $alignment->getMarginLeft();
            $indent = (Alignment::INDENT_NONE === $alignment->getIndent()) ? 0 : $alignment->getIndent();
            $paddingLeft = $marginLeft + $indent;

            if ($isSlideHolder) {
                if (false !== strpos($this->type, 'title')) {
                    $masterStyle = $this->slide->getTitleStyleAtLv($level);
                } else {
                    $masterStyle = $this->slide->getBodyStyleAtLv($level);
                }
                if (isset($masterStyle)) {
                    $bulletType = $masterStyle->getBulletStyle()->getBulletType();
                }

                $holderIdx = $this->phpShape->getPlaceholder()->getIdx();
                $layoutStyle = $this->slide->getLayout()->getHolderStyleAtLvl($holderIdx, $level);
                if (isset($layoutStyle) && Bullet::TYPE_NOTSET !== $layoutStyle->getBulletStyle()->getBulletType()) {
                    $bulletType = $layoutStyle->getBulletStyle()->getBulletType();
                }
            } else {
                $defListStyle = $this->slide->getDefaultListStyle();
                if (isset($defListStyle)) {
                    $style = $defListStyle->getStyleAtLvl($level);
                    if (isset($style)) {
                        $bulletType = $style->getBulletStyle()->getBulletType();
                    }
                }
            }

            $listStyle = $this->phpShape->getListStyle()->getStyleAtLvl($level);
            if (isset($listStyle)) {
                $bulletType = $listStyle->getBulletStyle()->getBulletType();
            }
            $paragraph->setListStyle($listStyle);
            $paraStyles = $paragraph->setData($phpParagraph, $isSlideHolder, $this->phpShape->isTextBox());

            if (Bullet::TYPE_NOTSET !== $phpParagraph->getBulletStyle()->getBulletType()) {
                $bulletType = $phpParagraph->getBulletStyle()->getBulletType();
            }

            if (Bullet::TYPE_NOTSET === $bulletType) {
                $bulletType = Bullet::TYPE_NONE;
            }

            if (Bullet::TYPE_NONE !== $bulletType && !$noText) {
                if (Bullet::TYPE_NOTSET === $preType || Bullet::TYPE_NONE === $preType || $level < $preLevel - 1) {
                    $preLevel = -1;
                    $container = $textBlock;
                    unset($listStack);
                    $listStack = array();
                }

                if (1 < $level && -1 === $preLevel) {
                    for ($i = 1; $i < $level; $i++) {
                        $list = ('numeric' === $bulletType) ? new OrderedList() : new BulletList();
                        $container->addContent($list);
                        $listItem = new ListItem();
                        $list->addContent($listItem);
                        $container = $listItem;
                    }
                    $preLevel = $level - 1;
                }

                if (1 === $level && !isset($paddingBase)) {
                    $paddingBase = $paddingLeft;
                }

                if (!isset($levelIndentMapping[$level])) {
                    $levelIndentMapping[$level] = $paddingLeft;
                } else if ($levelIndentMapping[$level] != $paddingLeft && 0 < $paddingLeft - $paddingBase) {
                    $paragraph->setAttr('paddingLeft', ($paddingLeft - $paddingBase) . 'px');
                }

                $subType = (Bullet::TYPE_NUMERIC === $bulletType) ? $phpParagraph->getBulletStyle()->getBulletNumericStyle() : '';

                if ($preLevel < $level) {
                    if (-1 !== $preLevel) {
                        array_push($listStack, $list);
                    }
                    $list = ('numeric' === $bulletType) ? new OrderedList() : new BulletList();
                    $container->addContent($list);
                } else if ($preLevel > $level) {
                    $list = array_pop($listStack);
                } else if (Bullet::TYPE_NUMERIC === $bulletType && $subType != $preSubType) {
                    if (1 === $level) {
                        $list = new OrderedList();
                        $textBlock->addContent($list);
                    } else {
                        $list = array_pop($listStack);
                    }
                }
                $preLevel = $level;
                $listItem = new ListItem();

                $marks = array();
                if ($this->slide && isset($paraStyles)) {
                    $this->slide->styleToMarks($paraStyles, $marks);
                }

                if (0 < count($marks)) {
                    $listItem->setAttr('listMarks', $marks);
                }
                $list->addContent($listItem);
                $container = $listItem;
            } else if (Bullet::TYPE_NONE == $bulletType) {
                if (1 < $level) {
                    $paragraph->setAttr('paddingLeft', ($level * 30) . 'px');
                } else if (Alignment::INDENT_NONE != $phpParagraph->getAlignment()->getIndent() && 0 < $phpParagraph->getAlignment()->getIndent()) {
                    $paragraph->setAttr('paddingLeft', Utils::getIndent($phpParagraph->getAlignment()->getIndent(), $levelIndentMapping));
                }
                $preLevel = -1;
                $container = $textBlock;
            }
            if (!isset($container)) {
                $container = $textBlock;
            }
            $preType = $bulletType;
            $preSubType = $subType;

            $container->addContent($paragraph);
        }


        if (0 === count($phpParagraphs)) {
            $paragraph = new Paragraph();
            $paragraph->setSlide($this->slide);
            $textBlock->addContent($paragraph);
        }

        $shape->addContent($textBlock);
        $this->addContent($shape);
    }

    protected function setShapePathConfig($shape)
    {
        $fillOpacity = 'fill-opacity';

        $strokeAttrs = new \stdClass();
        $strokeAttrs->stroke = true;
        $strokeAttrs->stroke_config = new \stdClass();
        $this->setBorder($this->phpShape->getBorder(), $strokeAttrs);
        $shape->setAttr('stroke', $strokeAttrs->stroke);
        $shape->setAttr('stroke_config', $strokeAttrs->stroke_config);

        $phpFill = $this->phpShape->getFill();
        if (isset($phpFill)) {
            $fill_config = new \stdClass();

            $fillType = $phpFill->getFillType();
            $shape->setAttr('fill', (Fill::FILL_NONE === $fillType) ? false : true);
            if (Fill::FILL_SOLID == $fillType) {
                $fillColor = $phpFill->getStartColor();
                $fill_config->fill = '#' . $this->slide->getColor($fillColor);
                $fill_config->$fillOpacity = $this->slide->getOpacity($fillColor);
                $shape->setAttr('fill_config', $fill_config);
            } else if (Fill::FILL_BLIP == $fillType) {
                global $synoTempPath;
                $id = rand();
                $name = rand();

                $imgPath = "{$synoTempPath}{$id}:{$name}";
                $handle = fopen($imgPath, 'wb', false);
                if (false !== $handle) {
                    fwrite($handle, $phpFill->getImageStringData());
                    fclose($handle);
                }
                $fill_config->fill = "$id";
                $fill_config->$fillOpacity = 1;
                $shape->setAttr('fill_config', $fill_config);
            }
        }
    }

    protected function setTextBlockVAlign($textBlock)
    {
        $phpAlignment = $this->phpShape->getAlignment();
        if (!is_null($phpAlignment)) {
            switch ($phpAlignment->getVertical()) {
                case 'ctr':
                    $alignment = 0;
                    break;
                case 'b':
                    $alignment = 1;
                    break;
                case 't':
                    $alignment = -1;
                default:
                    break;
            }
            if (isset($alignment)) {
                $textBlock->setAttr('vertical_align', $alignment);
            }
        }
    }

    private function setTextBlockPadding($textBlock)
    {
        $padding = new \stdClass();
        if (RichText::INSET_NONE !== $this->phpShape->getInsetTop()) {
            $padding->top = $this->phpShape->getInsetTop();
        }
        if (RichText::INSET_NONE !== $this->phpShape->getInsetRight()) {
            $padding->right = $this->phpShape->getInsetRight();
        }
        if (RichText::INSET_NONE !== $this->phpShape->getInsetBottom()) {
            $padding->bottom = $this->phpShape->getInsetBottom();
        }
        if (RichText::INSET_NONE !== $this->phpShape->getInsetLeft()) {
            $padding->left = $this->phpShape->getInsetLeft();
        }
        $textBlock->setAttr('padding', $padding);
    }
}

?>
