<?php
/**
 * This file is part of PHPWord - A pure PHP library for reading and writing
 * word processing documents.
 *
 * PHPWord is free software distributed under the terms of the GNU Lesser
 * General Public License version 3 as published by the Free Software Foundation.
 *
 * For the full copyright and license information, please read the LICENSE
 * file that was distributed with this source code. For the full list of
 * contributors, visit https://github.com/PHPOffice/PHPWord/contributors.
 *
 * @link        https://github.com/PHPOffice/PHPWord
 * @copyright   2010-2014 PHPWord contributors
 * @license     http://www.gnu.org/licenses/lgpl.txt LGPL version 3
 */

namespace PhpOffice\PhpWord\Writer\HTML_ProseMirror\Element;

use PhpOffice\PhpWord\Element\AbstractElement as Element;
use PhpOffice\PhpWord\Writer\AbstractWriter;

/**
 * Abstract HTML element writer
 *
 * @since 0.11.0
 */
abstract class AbstractElement
{
    /**
     * Parent writer
     *
     * @var \PhpOffice\PhpWord\Writer\AbstractWriter
     */
    protected $parentWriter;

    /**
     * Element
     *
     * @var \PhpOffice\PhpWord\Element\AbstractElement
     */
    protected $element;

    /**
     * Without paragraph
     *
     * @var bool
     */
    protected $withoutP = false;

    /**
     * previous writer
     *
     * @var \PhpOffice\PhpWord\Writer\HTML\Element
     */
    protected $prev_writer = null;

    /**
     * previous element
     *
     * @var \PhpOffice\PhpWord\Element\AbstractElement $element
     */
    protected $prev_element = null;

    /**
     * next element
     *
     * @var \PhpOffice\PhpWord\Element\AbstractElement $element
     */
    protected $next_element = null;

    /**
     * Write element
     */
    abstract public function write();

    /**
     * Create new instance
     *
     * @param \PhpOffice\PhpWord\Writer\AbstractWriter $parentWriter
     * @param \PhpOffice\PhpWord\Element\AbstractElement $element
     * @param bool $withoutP
     */
    public function __construct(AbstractWriter $parentWriter, Element $element, $withoutP = false)
    {
        $this->parentWriter = $parentWriter;
        $this->element = $element;
        $this->withoutP = $withoutP;
    }

    /**
     * Set without paragraph.
     *
     * @param bool $value
     * @return void
     */
    public function setWithoutP($value)
    {
        $this->withoutP = $value;
    }

    /**
     * Setup previous writer
     *
     * @param \PhpOffice\PhpWord\Writer\HTML\Element
     * @return void
     */
    public function setPrevWriter($writer)
    {
        $this->prev_writer = $writer;
    }

    /**
     * Setup previous element
     *
     * @param \PhpOffice\PhpWord\Element\AbstractElement $element
     * @return void
     */
    public function setPrevEl($element)
    {
        $this->prev_element = $element;
    }

    /**
     * Setup next element
     *
     * @param \PhpOffice\PhpWord\Element\AbstractElement $element
     * @return void
     */
    public function setNextEl($element)
    {
        $this->next_element = $element;
    }

    protected function generateRandomString($length = 10) {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }
}
