"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const node_1 = require("./node");
const html_serializer_1 = require("../html-serializer");
// FIXME: require() will not be compiled by tsc
let CSSStyleDeclaration = require('cssstyle').CSSStyleDeclaration;
class Element extends node_1.default {
    constructor(tagName, owner) {
        super(1 /* ELEMENT_NODE */, tagName.toUpperCase(), null);
        this.attributes = [];
        this.ownerDocument = owner;
        this._style = new CSSStyleDeclaration((newCssText) => {
            this.setAttribute('style', newCssText, true);
        });
    }
    get style() {
        return this._style;
    }
    get tagName() {
        return this.nodeName;
    }
    get children() {
        let ret = [];
        let child = this.firstChild;
        while (child) {
            if (child.nodeType === 1 /* ELEMENT_NODE */) {
                ret.push(child);
            }
            child = child.nextSibling;
        }
        return ret;
    }
    get firstElementChild() {
        let child = this.firstChild;
        while (child && child.nodeType !== 1 /* ELEMENT_NODE */) {
            child = child.nextSibling;
        }
        return child;
    }
    get nextElementSibling() {
        let next = this.nextSibling;
        while (next && next.nodeType !== 1 /* ELEMENT_NODE */) {
            next = next.nextSibling;
        }
        return next;
    }
    get innerHTML() {
        return (new html_serializer_1.default()).serializeChildren(this);
    }
    matches(selector) {
        return this.ownerDocument.nwmatcher.match(this, selector);
    }
    forEachElements(func) {
        let next = this.firstElementChild;
        while (next) {
            func(next);
            next.forEachElements(func);
            next = next.nextElementSibling;
        }
    }
    querySelectorAll(selector) {
        let arr = [];
        this.forEachElements((e) => {
            if (e.matches(selector)) {
                arr.push(e);
            }
        });
        return arr;
    }
    getAttribute(name) {
        const attributes = this.attributes;
        const n = name.toLowerCase();
        let attr;
        for (let i = 0, l = attributes.length; i < l; i++) {
            attr = attributes[i];
            if (attr.name === n) {
                return attr.value;
            }
        }
        return null;
    }
    hasAttribute(name) {
        const attributes = this.attributes;
        const n = name.toLowerCase();
        for (let i = 0, l = attributes.length; i < l; i++) {
            if (attributes[i].name === n) {
                return true;
            }
        }
        return false;
    }
    setAttribute(name, value, skip) {
        const attributes = this.attributes;
        const n = name.toLowerCase();
        let v;
        if (typeof value === 'string') {
            v = value;
        }
        else {
            v = '' + value;
        }
        let attr;
        for (let i = 0, l = attributes.length; i < l; i++) {
            attr = attributes[i];
            if (attr.name === n) {
                attr.value = v;
                break;
            }
        }
        if (!attr || attr.name !== n) {
            attributes.push({
                name: n,
                specified: true,
                value: v,
            });
        }
        if (n === 'style' && undefined === skip) {
            this._style.cssText = value;
        }
    }
    removeAttribute(name) {
        const n = name.toLowerCase();
        const attributes = this.attributes;
        if (n === 'style') {
            this._style.cssText = '';
        }
        for (let i = 0, l = attributes.length; i < l; i++) {
            const attr = attributes[i];
            if (attr.name === n) {
                attributes.splice(i, 1);
                return;
            }
        }
    }
    _cloneNode() {
        const node = new Element(this.tagName, this.ownerDocument);
        const attrs = node.attributes = [];
        for (const attr of this.attributes) {
            if (attr.name === 'style') {
                node._style.cssText = attr.value;
            }
            else {
                attrs.push({ name: attr.name, specified: attr.specified, value: attr.value });
            }
        }
        return node;
    }
}
exports.default = Element;
